#ifndef _BLUEIO_LIB_INTERFACE_H_
#define _BLUEIO_LIB_INTERFACE_H_

/***************************************************************************************************************************
	BLUEIO_lib_interface.h

	This file documents the 'C' interface to the BlueIO library.

	This library is implemented as BlueIO_lib.dll on Windows and as libBlueIO.dylib on the Mac.
	
***************************************************************************************************************************/
#ifdef TARGET_OS_LINUX
#ifdef __cplusplus
	#define BLUEIO_LIB_INTERFACE_DECL extern "C" __attribute__ ((visibility("default")))
#else
	#define BLUEIO_LIB_INTERFACE_DECL __attribute__ ((visibility("default")))
#endif
#elif TARGET_OS_MAC
#ifdef __cplusplus
	#define BLUEIO_LIB_INTERFACE_DECL extern "C"
#else
	#define BLUEIO_LIB_INTERFACE_DECL
#endif
#else
#ifdef __cplusplus
	#define BLUEIO_LIB_INTERFACE_DECL extern "C"
#else
	#define BLUEIO_LIB_INTERFACE_DECL
#endif
#endif

#ifndef GTYPE_NUMS
#define GTYPE_NUMS
#if defined(TARGET_OS_WIN) || defined(TARGET_OS_MAC) || defined(TARGET_OS_LINUX)
typedef short gtype_int16;
typedef unsigned short gtype_uint16;
typedef int gtype_int32;
typedef unsigned int gtype_uint32;
typedef long long gtype_int64;
typedef unsigned long long gtype_uint64;
typedef unsigned char gtype_bool;
typedef double gtype_real64;
typedef float gtype_real32;
#endif
#ifdef TARGET_OS_CTD
typedef short gtype_int16;
typedef unsigned short gtype_uint16;
typedef long gtype_int32;
typedef unsigned long gtype_uint32;
//typedef long long gtype_int64;
//typedef unsigned long long gtype_uint64;
typedef unsigned char gtype_bool;
#endif
#endif

#include "WDSSCmds.h"
#include "GSensorDDSMem.h"

#define BLUEIO_MAX_SIZE_DEVICE_NAME 220
typedef void *BLUEIO_LIBRARY_HANDLE;
typedef void *BLUEIO_DEVICE_HANDLE;
typedef void *BLUEIO_DEVICE_LIST_HANDLE;
typedef void *BLUEIO_PTR;

//The only BlueIO device currently supported is the WDSS.
#define BLUEIO_DEVTYPE_WDSS 6

#define BLUEIO_TIMEOUT_MS_DEFAULT 1000
#define BLUEIO_TIMEOUT_MS_READ_DDSMEMBLOCK 2000

/***************************************************************************************************************************
	Function Name: BlueIO_Init()
	
	Purpose:	Call BlueIO_Init() once before making any other BlueIO function calls.
				BlueIO_Init() and BlueIO_Uninit() should be called from the same thread.

				In principle, several different applications should be able to use the BlueIO library to access
				separate LabQuests concurrently. This is our design goal. However, this has not been tested much yet.

	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL BLUEIO_LIBRARY_HANDLE BlueIO_Init();

/***************************************************************************************************************************
	Function Name: BlueIO_Uninit()
	
	Purpose:	Call BlueIO_Uninit() once to 'undo' BlueIO_Init().
				BlueIO_Init() and BlueIO_Uninit() should be called from the same thread.

	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Uninit(BLUEIO_LIBRARY_HANDLE hLib);

/***************************************************************************************************************************
	Function Name: BlueIO_GetDLLVersion()
	
	Purpose:	This routine returns the major and minor version numbers for the instance of the BlueIO library that is
				currently running.

				If a function is not guaranteed to be present in all supported versions of the BlueIO library, then the line
				"Added in version 'major.minor'" will appear in the function description in this file.

				It is our intention that all versions of the BlueIO library created subsequent to a given version, will be
				backwards compatible with the older version. You should be able to replace an old version of the BlueIO library
				with a newer version and everything should still work without rebuilding your application.

				Note that version major2.minor2 is later than version major1.minor1 
				iff. ((major2 > major1) || ((major2 == major1) && (minor2 > minor1))).
				
				Backwards compatibility is definitely our intention, but we do not absolutely guarantee it. If you think
				that you have detected a backwards compatibility bug, then please report it to Vernier Software & Technology.
				Calling BlueIO_GetDLLVersion() from your application is a way to identify precisely which version of
				the BlueIO library you are actually using.

	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_GetDLLVersion(
	BLUEIO_LIBRARY_HANDLE hLib,	//[in] handle returned by BlueIO_Init()
	gtype_uint16 *pMajorVersion, //[out]
	gtype_uint16 *pMinorVersion); //[out]


/***************************************************************************************************************************
	Function Name: BlueIO_SearchForDevices()
	
	Purpose:	The BlueIO library maintains a separate list of available devices for each supported device type.
				BlueIO_SearchForDevices() updates the list for the specified device type.

				For each device list a signature is maintained that is incremented every time the list changes. The signature
				is implemented as a simple 32 bit counter that wraps from 0xffffffff to 0.
				
				Let int delta = sig2 - sig1. 
				If delta > 0, then sig2 is newer than sig1.
				If delta < 0, then sig2 is older than sig1.

	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_SearchForDevices(
	BLUEIO_LIBRARY_HANDLE hLib,		//[in] handle returned by BlueIO_Init()
	gtype_uint32 deviceType,		//[in] BLUEIO_DEVTYPE_...
	BLUEIO_PTR pParams,				//[in] ptr to context specific search params. Not currently used.
	gtype_uint32 *pDeviceListSignature);//[out] ptr to device list signature. Signatures are not fully implemented yet.

/***************************************************************************************************************************
	Function Name: BlueIO_OpenDeviceListSnapshot()
	
	Purpose:	The BlueIO library maintains a separate list of available devices for each supported device type.
				BlueIO_SearchForDevices() updates the list for the specified device type.
				Eventually, when BlueIO_EnableAutoDeviceDiscoveryAcrossCommTransport() is fully implemented, device lists
				will be fairly volatile. Until that happens, the only way to update a device list is to call
				BlueIO_SearchForDevices(). BlueIO_OpenDeviceListSnapshot() creates a snapshot copy of the device list for
				the specified device type. Use the handle returned by BlueIO_OpenDeviceListSnapshot() to walk the device
				list.

				A device is placed in the list snapshot even if it has already been opened.

				For each device list a signature is maintained that is incremented every time the list changes. The signature
				is implemented as a simple 32 bit counter that wraps from 0xffffffff to 0.
				
				Let int delta = sig2 - sig1. 
				If delta > 0, then sig2 is newer than sig1.
				If delta < 0, then sig2 is older than sig1.

	Return:		handle to device list snapshot iff successful, else NULL.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL BLUEIO_DEVICE_LIST_HANDLE BlueIO_OpenDeviceListSnapshot(
	BLUEIO_LIBRARY_HANDLE hLib,			//[in] handle returned by BlueIO_Init()
	gtype_uint32 deviceType,			//[in] BlueIO_DEVTYPE_...
	gtype_uint32 *pNumDevices,			//[out] ptr to storage loc for the number of devices in the list
	gtype_uint32 *pDeviceListSignature);//[out] ptr to device list signature. Signatures are not fully implemented yet.

/***************************************************************************************************************************
	Function Name: BlueIO_CloseDeviceListSnapshot()
	
	Purpose:	Close the list created by BlueIO_OpenDeviceListSnapshot().

	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_CloseDeviceListSnapshot(BLUEIO_DEVICE_LIST_HANDLE hDeviceList);

/***************************************************************************************************************************
	Function Name: BlueIO_DeviceListSnapshot_GetNthEntry()
	
	Purpose:	Return the Nth entry in the list created by BlueIO_OpenDeviceListSnapshot().
				A device is placed in the list snapshot even if it has already been opened.
				
				Pass the device name string placed in *pDevnameBuf to BlueIO_Device_Open() to open the device. Each
				device name string uniquely identifies the device, so you can determine if a device is already open
				by comparing *pDevnameBuf with the string returned by BlueIO_Device_GetOpenDeviceName() for each open
				device handle.

	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
#define BLUEIO_DEVICE_STATUS_MASK_OPEN 1
#define BLUEIO_DEVICE_STATUS_MASK_OPENED_BY_THIS_CLIENT 2
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_DeviceListSnapshot_GetNthEntry(
	BLUEIO_DEVICE_LIST_HANDLE hDeviceList,	//[in] handle returned by BlueIO_OpenDeviceListSnapshot()
	gtype_uint32 N,		//[in] index into list of known devices, 0 => first device in list.
	char *pDevnameBuf,	//[out] ptr to buffer to store device name string.
	gtype_uint32 bufSize,//[in] number of bytes in buffer pointed to by pDevnameBuf. Strlen(pDevnameBuf) < bufSize, because the string is NULL terminated.
						//Strlen(pDevnameBuf)is guaranteed to be < BLUEIO_MAX_SIZE_DEVICE_NAME.
	gtype_uint32 *pDeviceStatusMask);	//[out] BLUEIO_DEVICE_STATUS_MASK_... 

/***************************************************************************************************************************
	Function Name: BlueIO_Device_Open()
	
	Purpose:	Open a device with the name returned by BlueIO_DeviceListSnapshot_GetNthEntry.

				After establishing basic communication, BlueIO_Device_Open() sends a BLUEIO_CMD_ID_INIT command to the 
				BlueIO_DEVTYPE_LABPRO2 device. In response to BLUEIO_CMD_ID_INIT, all measurement buffers are cleared and each 
				channel is configured with default settings. No measurements will be taken until
				a BLUEIO_CMD_ID_START_MEASUREMENTS command is sent to the device.

				Currently, BlueIO_Device_Open() does not automatically detect what kinds of sensors are plugged in and it does
				not automatically read the DDS memory on the sensor. The application program has to do this manually.

				After calling BlueIO_Device_Open(), an application needs to perform the following operations to take 
				measurements:

				-	Call BlueIO_Device_SetMeasurementPeriod() to set the sampling period used for measurements.
				-	Send a BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK command to define which channels will take measurements in
						the next collection. If BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK is not sent, then
						all channels are enabled by default.
				-	Send a BLUEIO_CMD_ID_START_MEASUREMENTS command to start taking measurements.
				-	Call BlueIO_Device_ReadRawMeasurements() to get raw binary measurements. Call BlueIO_Device_ConvertToVoltage()
						and BlueIO_Device_CalibrateData() to convert raw measurements to cooked values. BlueIO library maintains a large
						(>= 10000 measurements) circular measurement buffer for each channel. If a collection exceeds 10000
						measurements, then the application needs to pull the measurements out of the buffer by calling
						BlueIO_Device_ReadRawMeasurements() before the buffer fills.
				-	Send a BLUEIO_CMD_ID_STOP_MEASUREMENTS command to stop taking measurements. Once BLUEIO_CMD_ID_STOP_MEASUREMENTS
						has been sent, the app can take its time about calling BlueIO_Device_ReadRawMeasurements() to pull measurements
						out of the measurement buffers. However, the app must empty the measurement buffers before sending
						BLUEIO_CMD_ID_START_MEASUREMENTS again.

				At any given time, a device is 'owned' by only one thread. The thread that calls this routine is the
				initial owner of the device. If an BlueIO() call is made from a thread that does not own the device object
				that is passed in, then the call will generally fail. To allow another thread to access a device,
				the owning thread must call BlueIO_Device_Unlock(), and then the new thread should call BlueIO_Device_Lock().

	Return:		handle to open device if successful, else NULL.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL BLUEIO_DEVICE_HANDLE BlueIO_Device_Open(
	BLUEIO_LIBRARY_HANDLE hLib,
	const char *pDeviceName);	//[in] NULL terminated string that uniquely identifies the device. See BlueIO_GetNthAvailableDeviceName().

/***************************************************************************************************************************
	Function Name: BlueIO_Device_Close()
	
	Purpose:	Close a specified device. After this routine runs, the hDevice handle is no longer valid.
	
				At any given time, a device is 'owned' by only one thread. The thread that called BlueIO_Device_Open() is the
				initial owner of the device. If an BlueIO() call is made from a thread that does not own the device object
				that is passed in, then the call will generally fail. To allow another thread to access a device,
				the owning thread must call BlueIO_Device_Unlock(), and then the new thread should call BlueIO_Device_Lock().

	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_Close(
	BLUEIO_DEVICE_HANDLE hDevice);//[in] handle to open device.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_GetOpenDeviceName()
	
	Purpose:	Get the unique device name string for a specified open device. This is the string returned by
				BlueIO_DeviceListSnapshot_GetNthEntry() and passed in to BlueIO_Device_Open().


	Return:		0 iff successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_GetOpenDeviceName(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	char *pDeviceName,			//[out] ptr to buffer to store device name string.
	gtype_uint32 bufSize);	//[in] number of bytes in buffer pointed to by pDeviceName. Strlen(pDeviceName) < bufSize, because the string is NULL terminated.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_Lock()
	
	Purpose:	Lock a specified device so that no other thread can access it. This routine will fail if the device
				is currently locked by another thread.

				BlueIO_Device_Lock() increments the lock count for the specified device by 1. In order for a second
				thread to acquire access to the device, the first thread must call BlueIO_Device_Unlock() until the lock
				count reaches 0. Note that the lock count is set to 1 by BlueIO_Device_Open().
				
				It is ok for the lock count to be greater than 1 when BlueIO_Device_Close() is called.

				This is an advanced routine that most users should not use. It is only necessary if an application
				needs to access a single device from multiple threads.
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_Lock(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	gtype_uint32 timeoutMs);	//[in] wait this long to acquire the lock before failing.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_Unlock()
	
	Purpose:	Decrement the lock count for the specified device by 1. This routine will fail if the device
				is not currently locked by the calling thread.

				In order for a second thread to acquire access to the device, the first thread must call 
				BlueIO_Device_Unlock() until the lock count reaches 0. Note that the lock count is set to 1 by 
				BlueIO_Device_Open().

				If the lock count reaches zero, it is important to make a call to BlueIO_Device_Lock() to lock the
				device. This will prevent multiple threads from simultaneously accessing the device, which can cause
				unpredictable results.
				
				It is ok for the lock count to be greater than 1 when BlueIO_Device_Close() is called.

				This is an advanced routine that most users should not use. It is only necessary if an application
				needs to access a single device from multiple threads.
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_Unlock(
	BLUEIO_DEVICE_HANDLE hDevice);//[in] handle to open device.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_ClearIO()
	
	Purpose:	Clear the input/output communication buffers for a specified device. 
				This also expties the BlueIO Measurement Buffer associated with the specified channel, which holds measurements 
				that have been retrieved from the device.
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_ClearIO(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel);		//[in] -1 => all channels.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_SendCmdAndGetResponse()
	
	Purpose:	Send a command to the specified device hardware and wait for a response. 

				Each device type has a command protocol that is unique to that device type. The command protocol used by
				BlueIO_DEVTYPE_LABPRO2_AUDIO devices is a subset of the protocol used by BlueIO_DEVTYPE_LABPRO2 devices. This
				protocol is documented in BlueIOSourceCmds.h .

				The command protocol used by BlueIO_DEVTYPE_LABPRO2_FILE_SERVER devices is documented in BlueIOFTLSourceCmds.h 

				Note that BlueIO_Device_SendCmdAndGetResponse() will fail if you send a BLUEIO_CMD_ID_START_MEASUREMENTS
				command while BlueIO_Device_GetNumMeasurementsAvailable() says measurements are available. 

				So...if you are restarting measurements, you should clear any old measurements in the BlueIO Measurement 
				Buffer first by reading in the measurements until the Buffer is empty, or you should call 
				BlueIO_Device_ClearIO().
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_SendCmdAndGetResponse(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	unsigned char cmd,			//[in] command code.
	BLUEIO_PTR pParams,			//[in] ptr to cmd specific parameter block, may be NULL. See GSkipCommExt.h.
	gtype_uint32 nParamBytes,	//[in] # of bytes in (*pParams).
	BLUEIO_PTR pRespBuf,		//[out] ptr to destination buffer, may be NULL.
	gtype_uint32 *pnRespBytes,	//[in, out] ptr to size of of pRespBuf buffer on input, size of response on output, may be NULL if pRespBuf is NULL.
	gtype_uint32 timeoutMs);	//[in] # of milliseconds to wait for a reply before giving up. Most devices should reply to almost all the 
							//currently defined commands within BLUEIO_TIMEOUT_MS_DEFAULT milliseconds. In fact, typical response
							//times are less than 50 milliseconds. See BLUEIO_TIMEOUT_MS_* definitions.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_SendCmd()
	
	Purpose:	BlueIO_Device_SendCmd() is an advanced function. You should usually use 
				BlueIO_Device_SendCmdAndGetResponse() instead. After calling BlueIO_Device_SendCmd(), you must call
				BlueIO_Device_GetNextResponse() before sending any more commands to the device.

				The main reason that BlueIO_Device_SendCmd() is made available to the user is to allow a program to send
				BLUEIO_CMD_ID_START_MEASUREMENTS commands to several different devices as close together as possible so that
				measurements start at about the same time on separate devices.
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_SendCmd(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	unsigned char cmd,	//[in] command code
	BLUEIO_PTR pParams,	//[in] ptr to cmd specific parameter block, may be NULL.
	gtype_uint32 nParamBytes,//[in] # of bytes in (*pParams).
	gtype_uint32 *pSignature);//[out] if not NULL, then put command signature emebedded in cmd blob here.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_GetNextResponse()
	
	Purpose:	BlueIO_Device_GetNextResponse() is an advanced function. You should usually use 
				BlueIO_Device_SendCmdAndGetResponse() instead. After calling BlueIO_Device_SendCmd(), you must call
				BlueIO_Device_GetNextResponse() before sending any more commands to the device.

	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_GetNextResponse(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	BLUEIO_PTR pRespBuf,			//[out] ptr to destination buffer, may be NULL.
	gtype_uint32 *pnRespBytes,  //[in, out] size of of dest buffer on input, size of response on output, may be NULL if pRespBuf is NULL.
	unsigned char *pCmd,		//[out] identifies which command this response is for. Must NOT be NULL!
	gtype_bool *pErrRespFlag,	//[out] flag indicating that the response contains error info. Must NOT be NULL!
	gtype_uint32 *pSignature,	//[in] if not NULL, then compare command signature emebedded in response blob with *pSignature.
	gtype_uint32 nTimeoutMs);	//[in] # of milliseconds to wait before giving up.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_GetMeasurementTick()
	
	Purpose:	The measurement period for devices is specified in discrete 'ticks', so the actual time between 
				measurements is an integer multiple of the tick time.
	
	Return:		If hDevice is not valid, then this routine returns -1.0, else the return value = the length of time
				in seconds between ticks.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_real64 BlueIO_Device_GetMeasurementTick(
	BLUEIO_DEVICE_HANDLE hDevice);	//[in] handle to open device.
	
/***************************************************************************************************************************
	Function Name: BlueIO_Device_GetMinimumMeasurementPeriod()
	
	Return:		If hDevice is not valid, then this routine returns -1.0, else the return value = minimum measurement
				period in seconds that is supported by the device.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_real64 BlueIO_Device_GetMinimumMeasurementPeriod(
	BLUEIO_DEVICE_HANDLE hDevice);	//[in] handle to open device.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_GetMaximumMeasurementPeriod()
	
	Return:		If hDevice is not valid, then this routine returns -1.0, else the return value = maximum measurement
				period in seconds that is supported by the device.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_real64 BlueIO_Device_GetMaximumMeasurementPeriod(
	BLUEIO_DEVICE_HANDLE hDevice);//[in] handle to open device.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_SetMeasurementPeriod()
	
	Purpose:	Set the measurement period to a specified number of seconds. The device will report measurements
				to the host computer at the measurement period interval once measurements have been started.
				These measurements are held in the BlueIO Measurement Buffer allocated for each channel. 
				See documentation for BlueIO_Device_GetNumMeasurementsAvailable().

				This routine sends BLUEIO_CMD_ID_SET_MEASUREMENT_PERIOD to the device with the parameter block configured 
				for the desiredPeriod.

				Because the measurement period is constrained to be a multiple of the tick returned by
				BlueIO_Device_GetMeasurementTick(), and because it must be 
					>= BlueIO_Device_GetMinimumMeasurementPeriod(), and
					<= BlueIO_Device_GetMaximumMeasurementPeriod(), 
				the actual base period is different than the desiredPeriod.

				You can determine the actual period by calling BlueIO_Device_GetMeasurementPeriod().

				This routine will fail if we are currently collecting measurements from the device. Note that collection
				is started by sending BLUEIO_CMD_ID_START_MEASUREMENTS to the device, and stopped by sending 
				BLUEIO_CMD_ID_STOP_MEASUREMENTS.

	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_SetMeasurementPeriod(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	gtype_real64 desiredPeriod,	//[in] desired measurement period in seconds.
	gtype_uint32 timeoutMs);	//[in] # of milliseconds to wait for a reply before giving up. BLUEIO_TIMEOUT_MS_DEFAULT is recommended.

/***************************************************************************************************************************
	Function Name: GoIO_Sensor_GetMeasurementPeriod()

	Purpose:	This routine sends BLUEIO_CMD_ID_GET_MEASUREMENT_PERIOD to the device to get the measurement period for
				the specified channel and then converts the result to seconds.
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_GetMeasurementPeriod(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	gtype_real64 *pPeriod,		//[out] ptr to loc to store period in seconds.
	gtype_uint32 timeoutMs);	//[in] # of milliseconds to wait for a reply before giving up. BLUEIO_TIMEOUT_MS_DEFAULT is recommended.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_GetNumMeasurementsAvailable()
	
	Purpose:	Report the number of measurements currently stored in the BlueIO Measurement Buffer for the
				specified channel.

				After BLUEIO_CMD_ID_START_MEASUREMENTS has been sent to the device, the device starts
				sending measurements to the host computer. These measurements are stored in the 
				BlueIO Measurement Buffer. A separate BlueIO Measurement Buffer is maintained for each
				channel of the device.

				Generally the device starts taking measurements immediately after receiving BLUEIO_CMD_ID_START_MEASUREMENTS.
				However, it may take approximately a 20 milliseconds for the data to come back from the device via a USB
				transport.

				Call BlueIO_Device_ReadRawMeasurements() to retrieve measurements from the BlueIO Measurement Buffer for a 
				specified channel. The BlueIO Measurement Buffer is guaranteed to hold at least 10000 measurements(per channel). 
				The buffer is circular, so if you do not service it often enough, the oldest measurements in the buffer 
				are lost. If you wish to capture all the measurements coming from the device, you must call 
				BlueIO_Device_ReadRawMeasurements() often enough so that the BlueIO_Device_GetNumMeasurementsAvailable() does not 
				reach 10000. On the other hand, we reserve the right to make the Measurement Buffer > 10000 measurements, so
				do not assume that you can empty the buffer simply by reading in 10000 measurements.

				Each of the following actions clears the BlueIO Measurement Buffer for a specified channel:
					1) Call BlueIO_Device_ReadRawMeasurements() with count set to BlueIO_Device_GetNumMeasurementsAvailable(), or
					2) Call BlueIO_Device_GetLatestRawMeasurement(), or
					3) Call BlueIO_Device_ClearIO().

				The BlueIO Measurement Buffers are empty after BlueIO_Device_Open() is called. They do not
				start filling up until BLUEIO_CMD_ID_START_MEASUREMENTS is sent to the device.

				Note that the device stops sending measurements to the host computer after
				BLUEIO_CMD_ID_STOP_MEASUREMENTS or BLUEIO_CMD_ID_INIT is sent, but sending these commands
				does not clear the BlueIO Measurement Buffers.

	Return:		number of measurements currently stored in the BlueIO Measurement Buffer for the specified channel. 

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_GetNumMeasurementsAvailable(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel);		//[in]

/***************************************************************************************************************************
	Function Name: BlueIO_Device_ReadRawMeasurements()
	
	Purpose:	Retrieve measurements from the BlueIO Measurement Buffer for a specified channel. The measurements reported
				by this routine are actually removed from the BlueIO Measurement Buffer.

				After BLUEIO_CMD_ID_START_MEASUREMENTS has been sent to the device, the device starts
				sending measurements to the host computer. These measurements are stored in the 
				BlueIO Measurement Buffer. A separate BlueIO Measurement Buffer is maintained for each
				channel of the device.
				See the description of BlueIO_Device_GetNumMeasurementsAvailable().

				Assuming that the current measurement period is Tp, then the first measurement in a
				channel's BlueIO Measurement Buffer occurred at time 0, the next measurement occurred at time Tp,
				then time 2*Tp, 3*Tp, and so on. If several different channels are sampled at the same rate, then
				at times n*Tp, the separate channels are sampled concurrently. In fact, if the hardware is configured
				two measure two different channels at the same time, the measurements occur several microseconds apart,
				but we generally report these measurements to have occurred at the same time.

	Return:		number of measurements retrieved from the BlueIO Measurement Buffer. This routine
				returns immediately, so the return value may be less than maxCount.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_ReadRawMeasurements(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,			//[in]
	gtype_int32 *pMeasurementsBuf,	//[out] ptr to loc to store measurements.
	gtype_uint32 maxCount);	//[in] maximum number of measurements to copy to pMeasurementsBuf. See warning above.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_GetLatestRawMeasurement(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel);				//[in]

BLUEIO_LIB_INTERFACE_DECL gtype_real64 BlueIO_Device_ConvertToVoltage(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,			//[in]
	gtype_int32 rawMeasurement); //[in] raw measurement obtained from BlueIO_Device_GetLatestRawMeasurement() or 
								//BlueIO_Device_ReadRawMeasurements().

BLUEIO_LIB_INTERFACE_DECL gtype_real64 BlueIO_Device_CalibrateData(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	gtype_real64 volts);		//[in] voltage value obtained from BlueIO_Device_ConvertToVoltage();

BLUEIO_LIB_INTERFACE_DECL gtype_real64 BlueIO_Device_CalibrateData2(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	gtype_int32 rawMeasurement);//[in] raw measurement obtained from BlueIO_Device_GetLatestRawMeasurement() or 
								//BlueIO_Device_ReadRawMeasurements().

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_GetChannelRange(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	gtype_uint32 *pMinChannel,	//[out]
	gtype_uint32 *pMaxChannel);	//[out]

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_NVMemBlk_GetAddrRange(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,		//[in]
	gtype_uint32 *pMinAddr,		//[out]
	gtype_uint32 *pMaxAddr);	//[out]

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_NVMemBlk_Read(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,		//[in]
	BLUEIO_PTR pDestBuf,
	gtype_uint32 addr,
	gtype_uint32 numBytesToRead,
	gtype_uint32 timeoutMs);		//[in] # of milliseconds to wait for a reply before giving up.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_NVMemBlk_Write(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,		//[in]
	BLUEIO_PTR pSrcBuf,
	gtype_uint32 addr,
	gtype_uint32 numBytesToWrite,
	gtype_uint32 timeoutMs);		//[in] # of milliseconds to wait for a reply before giving up.

/***************************************************************************************************************************/
/***************************************************************************************************************************/
/***************************************************************************************************************************
BlueIO_Device_DDSMem_* routines:

The BlueIO device object maintains a list of SensorDDSRecords of type GSensorDDSRec, one for each channel. 
The fields in this record describe the sensor plugged into the channel.
The BlueIO_Device_DDSMem_* routines allow an application program to access the SensorDDSRecord.
See GSensorDDSMem.h for the declaration of GSensorDDSRec.

Sensors come in 3 basic flavors:
1) Smart sensors,
2) AutoId capable sensors that are not smart, and
3) Legacy sensors.

Smart sensors store the entire 128 byte GSensorDDSRec record on the actual sensor hardware. This allows calibration
data that is specific to a given sensor to be stored on the sensor. Smart sensors report an AutoId number
in response to the BLUEIO_CMD_ID_GET_SENSOR_ID command that is >= kSensorIdNumber_FirstSmartSensor. 

If a sensor reports an AutoId >= kSensorIdNumber_FirstSmartSensor, then BlueIO_Device_Open() will get the GSensorDDSRec 
info from the sensor hardware and store in the BlueIO sensor object's SensorDDSRecord. 

Note that Go! Motion pretends to be smart: its id is > kSensorIdNumber_FirstSmartSensor, and BlueIO_Device_Open() sets 
up its GSensorDDSRec info with calibrations for meters and feet. However, there is no way to store a modified
GSensorDDSRec record on the sensor permanently, which truly smart sensors do support.

AutoId capable sensors that are not smart report an AutoId value > 0 and < kSensorIdNumber_FirstSmartSensor.

Legacy (aka Dumb) sensors report an AutoId = 0.

Non smart sensors just get default values for most of the fields in the SensorDDSRecord. The default calibration 
parameters are configured so that BlueIO_Device_CalibrateData() just reports volts.

For all 3 types of sensors, SensorDDSRecord.SensorNumber = AutoId after BlueIO_Device_Open(). However, for
legacy sensors that means SensorDDSRecord.SensorNumber will be 0 which is not informative. In fact, Go! Link
reports an AutoID of 0 even if no sensor is plugged in!

All the BlueIO_Device_DDSMem_* routines return 0 if successful, or a negative number if an error occurs. 
The BlueIO_Device_DDSMem_* routines that do not take a timeout parameter will not fail if hDevice is valid and owned by
the calling thread.

****************************************************************************************************************************/
/***************************************************************************************************************************/
/***************************************************************************************************************************/

/***************************************************************************************************************************
	Function Name: BlueIO_Device_DDSMem_WriteRecord()
	
	Purpose:	Write the SensorDDSRecord into a smart sensor's DDS memory hardware. 
				Before the record is written to the sensor, SensorDDSRecord.Checksum is recalculated.
				Before the record is written to the sensor, a copy of the record is marshalled into little endian format,
				and then this copy is actually written to the sensor.

				This routine sends BLUEIO_CMD_ID_WRITE_LOCAL_NV_MEM_*(Go! Temp) or BLUEIO_CMD_ID_WRITE_REMOTE_NV_MEM_*(Go! Link)
				commands to the device. To alter DDS memory on the sensor, we recommend using 
				BlueIO_Device_DDSMem_WriteRecord rather than the low level BLUEIO_CMD_ID_WRITE commands.

				WARNING: Be careful about using this routine. Changing a smart sensor's DDS memory can cause the sensor
				to stop working with Logger Pro.
	
	Return:		0 if successful, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_WriteRecord(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	gtype_uint32 timeoutMs);//[in] # of milliseconds to wait for a reply before giving up. BLUEIO_TIMEOUT_MS_WRITE_DDSMEMBLOCK is recommended.


/***************************************************************************************************************************
	Function Name: BlueIO_Device_DDSMem_ReadRecord()
	
	Purpose:	Read the SensorDDSRecord from a smart sensor's DDS memory hardware. 
				After the record is read from the sensor, it is unmarshalled from little endian format into processor
				specific format.

				This routine sends BLUEIO_CMD_ID_READ_LOCAL_NV_MEM(Go! Temp) or BLUEIO_CMD_ID_READ_REMOTE_NV_MEM(Go! Link)
				commands to the device. BlueIO_Device_Open() automatically calls this routine, so you usually do not
				need to.
	
	Return:		0 if successful, 
				else if data validation fails then -2,
				else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_ReadRecord(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	gtype_bool strictDDSValidationFlag,//[in] insist on exactly valid checksum if 1, else use a more lax validation test.
	gtype_uint32 timeoutMs);//[in] # of milliseconds to wait for a reply before giving up. BLUEIO_TIMEOUT_MS_READ_DDSMEMBLOCK is recommended.
						   //Note that BLUEIO_TIMEOUT_MS_READ_DDSMEMBLOCK is much longer than the typical time required which is < 500 ms.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_DDSMem_SetRecord()
	
	Purpose:	Update the SensorDDSRecord. 
				This routine does not send data to the sensor's DDS memory hardware - use BlueIO_Device_DDSMem_WriteRecord for that.

	Return:		0 if hDevice is valid, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetRecord(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	const GSensorDDSRec *pRec);	//[in] ptr to data to copy into the SensorDDSRecord.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_DDSMem_GetRecord()
	
	Purpose:	Get a copy of the SensorDDSRecord. 
				This routine does not read data from the sensor's DDS memory hardware - use BlueIO_Device_DDSMem_ReadRecord for that.

	Return:		0 if hDevice is valid, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetRecord(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	GSensorDDSRec *pRec);		//[out] ptr to dest buf to copy the SensorDDSRecord into.

/***************************************************************************************************************************
	Function Name: BlueIO_Device_DDSMem_CalculateChecksum()
	
	Return:		0 if hDevice is valid, else -1.

****************************************************************************************************************************/
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_CalculateChecksum(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	unsigned char *pChecksum);	//[out] ptr to checksum calculated by XOR'ing the first 127 bytes of the SensorDDSRecord.

/***************************************************************************************************************************/
/***************************************************************************************************************************/
/***************************************************************************************************************************
Languages that can handle GSensorDDSRec can access the DDS fields using BlueIO_Device_DDSMem_SetRecord and GetRecord.
Languages such as Basic are unlikely to be able to handle a complex record like GSensorDDSRec, especially because of
byte alignment issues. The individual field level API's defined below provide an alternate way to access the SensorDDSRecord.
****************************************************************************************************************************/
/***************************************************************************************************************************/
/***************************************************************************************************************************/

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetMemMapVersion(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char MemMapVersion);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetMemMapVersion(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char *pMemMapVersion);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetSensorNumber(
	BLUEIO_DEVICE_HANDLE hDevice,	//[in] handle to open device.
	signed char channel,				//[in]
	unsigned char SensorNumber);//[in]

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetSensorNumber(
	BLUEIO_DEVICE_HANDLE hDevice,		//[in] handle to open device. 
	signed char channel,				//[in]
	unsigned char *pSensorNumber,	//[out] ptr to SensorNumber.
	gtype_uint32 timeoutMs);//[in] # of milliseconds to wait for a reply before giving up. BLUEIO_TIMEOUT_MS_DEFAULT is recommended.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetSerialNumber(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char leastSigByte_SerialNumber,
	unsigned char midSigByte_SerialNumber,
	unsigned char mostSigByte_SerialNumber);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetSerialNumber(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char *pLeastSigByte_SerialNumber,
	unsigned char *pMidSigByte_SerialNumber,
	unsigned char *pMostSigByte_SerialNumber);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetLotCode(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char YY_BCD,
	unsigned char WW_BCD);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetLotCode(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char *pYY_BCD,
	unsigned char *pWW_BCD);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetManufacturerID(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char ManufacturerID);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetManufacturerID(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	unsigned char *pManufacturerID);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetLongName(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,				//[in]
	const char *pLongName);//[in] ptr to NULL terminated string.
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetLongName(
	BLUEIO_DEVICE_HANDLE hDevice,			//[in] handle to open device. 
	signed char channel,						//[in]
	char *pLongName,					//[out] ptr to buffer for NULL terminated output string.
	gtype_uint16 maxNumBytesToCopy);	//[in] size of pLongName buffer.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetShortName(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,			//[in]
	const char *pShortName);//[in] ptr to NULL terminated string.
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetShortName(
	BLUEIO_DEVICE_HANDLE hDevice,			//[in] handle to open device. 
	signed char channel,						//[in]
	char *pShortName,					//[out] ptr to buffer for NULL terminated output string. 
	gtype_uint16 maxNumBytesToCopy);	//[in] size of pShortName buffer.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetUncertainty(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char Uncertainty);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetUncertainty(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pUncertainty);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetSignificantFigures(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char SignificantFigures);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetSignificantFigures(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pSignificantFigures);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetCurrentRequirement(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char CurrentRequirement);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetCurrentRequirement(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pCurrentRequirement);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetAveraging(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char Averaging);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetAveraging(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pAveraging);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetMinSamplePeriod(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 MinSamplePeriod);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetMinSamplePeriod(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 *pMinSamplePeriod);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetTypSamplePeriod(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 TypSamplePeriod);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetTypSamplePeriod(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 *pTypSamplePeriod);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetTypNumberofSamples(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_uint16 TypNumberofSamples);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetTypNumberofSamples(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_uint16 *pTypNumberofSamples);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetWarmUpTime(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_uint16 WarmUpTime);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetWarmUpTime(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_uint16 *pWarmUpTime);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetExperimentType(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char ExperimentType);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetExperimentType(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pExperimentType);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetOperationType(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char OperationType);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetOperationType(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pOperationType);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetCalibrationEquation(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	char CalibrationEquation);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetCalibrationEquation(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	char *pCalibrationEquation);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetYminValue(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 YminValue);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetYminValue(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 *pYminValue);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetYmaxValue(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 YmaxValue);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetYmaxValue(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	gtype_real32 *pYmaxValue);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetYscale(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char Yscale);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetYscale(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pYscale);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetHighestValidCalPageIndex(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char HighestValidCalPageIndex);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetHighestValidCalPageIndex(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pHighestValidCalPageIndex);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetActiveCalPage(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char ActiveCalPage);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetActiveCalPage(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pActiveCalPage);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetCalPage(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char CalPageIndex,
	gtype_real32 CalibrationCoefficientA,
	gtype_real32 CalibrationCoefficientB,
	gtype_real32 CalibrationCoefficientC,
	const char *pUnits);//[in] ptr to NULL terminated string.
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetCalPage(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char CalPageIndex,
	gtype_real32 *pCalibrationCoefficientA,
	gtype_real32 *pCalibrationCoefficientB,
	gtype_real32 *pCalibrationCoefficientC,
	char *pUnits,						//[out] ptr to buffer for NULL terminated output string. 
	gtype_uint16 maxNumBytesToCopy);	//[in] size of pUnits buffer.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_SetChecksum(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char Checksum);
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Device_DDSMem_GetChecksum(
	BLUEIO_DEVICE_HANDLE hDevice,
	signed char channel,						//[in]
	unsigned char *pChecksum);

#define BLUEIO_TRACE_SEVERITY_LOWEST 1
#define BLUEIO_TRACE_SEVERITY_LOW 10
#define BLUEIO_TRACE_SEVERITY_MEDIUM 50
#define BLUEIO_TRACE_SEVERITY_HIGH 100
BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_SetDebugTraceThreshold(
	BLUEIO_LIBRARY_HANDLE hLib,
	gtype_int32 threshold);//[in] Only trace messages marked with a severity >= threshold are actually sent to the debug output.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_GetDebugTraceThreshold(
	BLUEIO_LIBRARY_HANDLE hLib,
	gtype_int32 *pThreshold);//[out]

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_SetIOTraceEnableFlag(
	BLUEIO_DEVICE_HANDLE hDevice,
	gtype_bool flag);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_GetIOTraceEnableFlag(
	BLUEIO_DEVICE_HANDLE hDevice,
	gtype_bool *pFlag);

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_GetNumInputTraceBytesAvailable(
	BLUEIO_DEVICE_HANDLE hDevice);//[in] handle to open device.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_ReadInputTraceBytes(
	BLUEIO_DEVICE_HANDLE hDevice,		//[in] handle to open device.
	unsigned char *pInputTraceBuf,	//[out] ptr to loc to store trace info.
	gtype_int32 maxCount);	//[in] maximum number of bytes to copy into pInputTraceBuf.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_GetNumOutputTraceBytesAvailable(
	BLUEIO_DEVICE_HANDLE hDevice);//[in] handle to open device.

BLUEIO_LIB_INTERFACE_DECL gtype_int32 BlueIO_Diags_ReadOutputTraceBytes(
	BLUEIO_DEVICE_HANDLE hDevice,		//[in] handle to open device.
	unsigned char *pOutputTraceBuf,	//[out] ptr to loc to store trace info.
	gtype_int32 maxCount);	//[in] maximum number of bytes to copy into pOutputTraceBuf.

#endif //_BlueIO_LIB_INTERFACE_H_
