#ifndef _WDSS_CMDS_H_
#define _WDSS_CMDS_H_

#include "BlueIOCmds.h"

/***************************************************************************************************/
//
// This file contains declarations for parameter and response structures used by the function 
// BlueIO_SendCmdAndGetResponse().
//
// BlueIO_SendCmdAndGetResponse() is used to send commands to the WDSS.
// The WDSS supports commands with the prefixes BLUEIO_CMD_ID_ and WDSS_CMD_ID .
//
/***************************************************************************************************/
#define WDSS_CHANNEL_TIME 0
#define WDSS_CHANNEL_FORCE_LOW 1	// App should never use this channel (use combined force all the time!)
#define WDSS_CHANNEL_FORCE_HIGH 2	// ditto
#define WDSS_CHANNEL_BAROMETER 3
#define WDSS_CHANNEL_ACCEL_Y 4
#define WDSS_CHANNEL_ACCEL_X 5
#define WDSS_CHANNEL_ACCEL_Z 6
#define WDSS_CHANNEL_COMBINED_FORCE 7	// "Fake" sensor that combines the low and high force sensors
#define WDSS_NUM_CHANNELS 8
#define WDSS_NUM_PHYSICAL_SENSORS 6
#define WDSS_NUM_LOGICAL_SENSORS 5

//Use the following values with BLUEIO_CMD_ID_SET_SENSOR_CHANNEL_ENABLE_MASK.
#define WDSS_CHANNEL_MASK_COMBINED_FORCE 6
#define WDSS_CHANNEL_MASK_BAROMETER 8
#define WDSS_CHANNEL_MASK_ACCEL_Y 0x10
#define WDSS_CHANNEL_MASK_ACCEL_X 0x20
#define WDSS_CHANNEL_MASK_ACCEL_Z 0x40


#define WDSS_BLUETOOTH_LED 1
//WDSS_BLUETOOTH_LED is NOT a valid value for the BlueIOSetLedStateParams.LEDchannel parameter for BLUEIO_CMD_ID_SET_LED_STATE.
//WDSS_COLLECTION_LED must  be used for the BlueIOSetLedStateParams.LEDchannel parameter for BLUEIO_CMD_ID_SET_LED_STATE.
#define WDSS_COLLECTION_LED 2

#define SENSOR_NV_MEM_BLOCK_SIZE 128
#define APP_REMOTE_HEADER_MEM_BLOCK_SIZE (24*1024) // 24K for application memory to store SRemoteColumnInfo structs in

#define WDSS_MAX_REMOTE_RUNS 20 // Max number of remote runs
#define WDSS_NUM_BYTES_PER_RAW_VALUE 2 // Each sample taken requires 2 bytes (12 bit A2D)
#define WDSS_REMOTE_HEADER_SIZE 256 // The header for a remote data run takes 256 bytes
#define WDSS_REMOTE_SLOT_SIZE (15*256) // Each memory "slot" is this many bytes
#define WDSS_NUM_REMOTE_SLOTS 128 // There are this many slots available to store remote data
#define WDSS_TOTAL_REMOTE_MEMORY (WDSS_REMOTE_SLOT_SIZE * WDSS_NUM_REMOTE_SLOTS) // How much total memory is available for remote runs?
#define WDSS_MAX_SINGLE_RUN_SIZE (WDSS_TOTAL_REMOTE_MEMORY/2) // How many bytes the largest possible remote run can handle

#define WDSS_ERROR_MASK_X1_FLASH_CATALOG_WRITE_FAILURE 0x01
#define WDSS_ERROR_MASK_X1_FLASH_RUN_HEADER_WRITE_FAILURE 0x02
#define WDSS_ERROR_MASK_X1_FLASH_UNKNOWN_ERROR 0x04
#define WDSS_ERROR_MASK_X1_CONFIGURE_RADIO_FAILURE 0x08
#define WDSS_ERROR_MASK_X1_ZERO_ALTIMETER_FAILURE 0x10
#define WDSS_ERROR_MASK_X1_FIFO_OVERFLOW1 0x20
#define WDSS_ERROR_MASK_X1_FIFO_OVERFLOW2 0x40

#define WDSS_STATUS_MASK_X2_BUTTON_PRESSED 0x01
#define WDSS_STATUS_MASK_X2_ARCHIVE_MGR_BUSY 0x02
#define WDSS_STATUS_MASK_X2_ARCHIVE_DIRTY 0x04

#define WDSS_STATUS_ERROR_ARCHIVE_BUSY 0x50
#define WDSS_STATUS_ERROR_RAM_TEST_FAILURE 0x51

#define WDSS_CMD_ID_GET_ARCHIVE_CATALOG 0x40
#define WDSS_CMD_ID_CONFIGURE_RADIO 0x41
#define WDSS_CMD_ID_TEST_RAM 0x42
#define WDSS_CMD_ID_TEST_RAM2 0x43

//
/***************************************************************************************************/
//

// Make sure structures get packed on 1-byte boundaries.  
// These preprocessor commands are in a few places throughout the code so if you change one, change them all!
#if defined TARGET_OS_WIN
	#pragma pack(push)
	#pragma pack(1)
#elif (defined TARGET_OS_MAC) || (defined TARGET_OS_LINUX)
	#pragma pack(1)
#endif

#ifndef GTYPE_NUMS
#define GTYPE_NUMS
#if defined(TARGET_OS_WIN) || defined(TARGET_OS_MAC) || defined(TARGET_OS_LINUX)
typedef short gtype_int16;
typedef unsigned short gtype_uint16;
typedef int gtype_int32;
typedef unsigned int gtype_uint32;
typedef long long gtype_int64;
typedef unsigned long long gtype_uint64;
typedef unsigned char gtype_bool;
typedef double gtype_real64;
typedef float gtype_real32;
#endif
#ifdef TARGET_OS_CTD
typedef short gtype_int16;
typedef unsigned short gtype_uint16;
typedef long gtype_int32;
typedef unsigned long gtype_uint32;
//typedef long long gtype_int64;
//typedef unsigned long long gtype_uint64;
typedef unsigned char gtype_bool;
#endif
#endif

/***************************************************************************************************/
//BLUEIO_CMD_ID_GET_STATUS:

typedef struct
{
	BlueIOGetStatusCmdResponsePayload base;
	unsigned char statusX1;	//See WDSS_ERROR_MASK_X1_*.
	unsigned char statusX2;	//See WDSS_STATUS_MASK_X2_*.
	unsigned char archiveMgrBusyProgressPercentage;	//only valid if (statusX2 & WDSS_STATUS_MASK_X2_ARCHIVE_MGR_BUSY) != 0.
	unsigned char archiveCleanupProgressPercentage;	//only valid if (statusX2 & WDSS_STATUS_MASK_X2_ARCHIVE_DIRTY) != 0.
	unsigned char diags[30];
} WDSSGetStatusCmdResponsePayload; //This is the response payload returned by GetNextResponse() after sending BLUEIO_CMD_ID_GET_STATUS.

/***************************************************************************************************/
//WDSS_CMD_ID_GET_ARCHIVE_CATALOG:
typedef struct			//All the multibyte fields in this struct come back from the device in big endian format.
{						//This is not intended to be used by apps - it is provided for diagnostic reasons.
	gtype_uint32 nextRunId;
	gtype_uint16 subRecId;
	gtype_uint32 key;
	unsigned char reservedSlotMask[16];
	unsigned char committedSlotMask[16];
	unsigned char numSlotsReserved;
	unsigned char numSlotsCommitted;
	unsigned char lastReservedSlotId;
	unsigned char numRuns;
	unsigned char firstSlots[20];
	gtype_uint32 runIds[20];
} WDSSArchiveCatalog;

//This is the response payload returned by GetNextResponse() after sending WDSS_CMD_ID_GET_ARCHIVE_CATALOG.
typedef WDSSArchiveCatalog WDSSGetArchiveCatalogCmdResponsePayload;

/***************************************************************************************************/
//WDSS_CMD_ID_CONFIGURE_RADIO:

typedef struct
{
	unsigned char opcode;
	unsigned char lsbyteTimeoutMs;	//Timeout measured down on the device when programming the radio submodule.
	unsigned char msbyteTimeoutMs;
	unsigned char dataBlockLength;
	unsigned char dataBlock[1];	//Actual length is dataBlockLength. 
} WDSSConfigureRadioParams; //Parameter block passed into SendCmd() with WDSS_CMD_ID_CONFIGURE_RADIO.

//This is the response payload returned by GetNextResponse() after sending WDSS_CMD_ID_CONFIGURE_RADIO.
typedef WDSSConfigureRadioParams WDSSConfigureRadioCmdResponsePayload;

/***************************************************************************************************/
//WDSS_CMD_ID_TEST_RAM:

//Note that SendCmdAndGetResponse() does not return kResponse_Error if a ram test fails. This is because
//we only report a simple single byte error code payload when kResponse_Error is returned.
//To support extended error info, SendCmdAndGetResponse returns kResponse_OK and detailed results are placed
//in WDSSTestRamCmdResponsePayload.
//kResponse_Error is returned if a bad parameter is sent.

typedef struct
{
	unsigned char lsbyteLswordAddr;	//Valid addr range is 0 to 0x3ffff.
	unsigned char msbyteLswordAddr;
	unsigned char lsbyteMswordAddr;
	unsigned char msbyteMswordAddr;
	unsigned char lsByteCount;
	unsigned char msByteCount;
} WDSSTestRamParams;

typedef struct
{
	unsigned char status;	//BLUEIO_STATUS_SUCCESS or WDSS_STATUS_ERROR_RAM_TEST_FAILURE;
	unsigned char lsbyteLswordAddr;
	unsigned char msbyteLswordAddr;
	unsigned char lsbyteMswordAddr;
	unsigned char msbyteMswordAddr;
	unsigned char byteWritten;
	unsigned char byteRead;
} WDSSTestRamCmdResponsePayload;

/***************************************************************************************************/
//WDSS_CMD_ID_TEST_RAM2:

//Note that SendCmdAndGetResponse() does not return kResponse_Error if a ram test fails. This is because
//we only report a simple single byte error code payload when kResponse_Error is returned.
//To support extended error info, SendCmdAndGetResponse returns kResponse_OK and detailed results are placed
//in WDSSTestRam2CmdResponsePayload.

//This command does not take parameters.

typedef WDSSTestRamCmdResponsePayload WDSSTestRam2CmdResponsePayload;

/***************************************************************************************************/

// Restore the old packing scheme
// These commands are in a few places throughout the code so if youchange one, change them all!
#if defined TARGET_OS_WIN
	#pragma pack(pop)
#elif (defined TARGET_OS_MAC) || (defined TARGET_OS_LINUX)
	#pragma pack()
#endif

#endif 